# Chinese lexemes common.py
lexemes = {
    "app_title": "Notolog 编辑器",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "文件过滤器字段",

    "menu_action_copy_file_path": "复制文件路径",
    "menu_action_rename": "重命名",
    "menu_action_delete": "删除",
    "menu_action_delete_completely": "彻底删除",
    "menu_action_restore": "恢复",
    "menu_action_create_new_dir": "创建新目录",

    "dialog_file_rename_title": "重命名文件",
    "dialog_file_rename_field_label": "输入新的文件名",
    "dialog_file_rename_button_ok": "重命名",
    "dialog_file_rename_warning_exists": "同名文件已经存在",

    "dialog_file_delete_title": "删除文件",
    "dialog_file_delete_text": "删除文件 \"{file_name}\"?",
    "dialog_file_delete_completely_title": "彻底删除文件",
    "dialog_file_delete_completely_text": "彻底删除文件 \"{file_name}\" 吗？",
    "dialog_file_delete_error": "无法删除文件，发生错误",
    "dialog_file_delete_error_not_found": "文件未找到",

    "dialog_file_restore_title": "恢复文件",
    "dialog_file_restore_text": "恢复文件 \"{file_name}\"?",
    "dialog_file_restore_error": "无法恢复文件，发生错误",
    "dialog_file_restore_warning_exists": "{file_name} 名称的文件已存在",

    "dialog_create_new_dir_title": "创建新目录",
    "dialog_create_new_dir_label": "新目录名称",
    "dialog_create_new_dir_input_placeholder_text": "输入目录名称",
    "dialog_create_new_dir_button_ok": "创建",
    "dialog_create_new_dir_button_cancel": "取消",
    "dialog_create_new_dir_warning_empty_title": "新目录名称错误",
    "dialog_create_new_dir_warning_empty_text": "目录名称不能为空",
    "dialog_create_new_dir_warning_too_long_title": "新目录名称错误",
    "dialog_create_new_dir_warning_too_long_text": "目录名称太长；最大允许 {symbols} 个字符！",
    "dialog_create_new_dir_error_existed": "目录已存在",
    "dialog_create_new_dir_error": "无法创建目录。请确保目标目录 {base_dir} 可写",

    "dialog_message_box_title": "消息",
    "dialog_message_box_button_ok": "关闭",

    "action_new_file_first_line_template_text": "新文档",
    "action_open_file_dialog_caption": "打开文件",
    "action_save_as_file_dialog_caption": "保存文件",

    "dialog_save_empty_file_title": "保存空文件",
    "dialog_save_empty_file_text": "允许保存空内容的文件吗？",

    "dialog_encrypt_file_title": "加密文件",
    "dialog_encrypt_file_text": "加密文件 \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "文件已经加密!",
    "dialog_encrypt_file_rewrite_existing_title": "重写现有文件",
    "dialog_encrypt_file_rewrite_existing_text": "重写现有文件 \"{file_path}\"?",

    "dialog_decrypt_file_title": "解密文件",
    "dialog_decrypt_file_text": "解密文件 \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "文件未加密!",
    "dialog_decrypt_file_rewrite_existing_title": "重写现有文件",
    "dialog_decrypt_file_rewrite_existing_text": "重写现有文件 \"{file_path}\"?",

    "dialog_encrypt_new_password_title": "新密码",
    "dialog_encrypt_new_password_label": "密码：",
    "dialog_encrypt_new_password_input_placeholder_text": "输入新密码",
    "dialog_encrypt_new_password_hint_label": "提示：",
    "dialog_encrypt_new_password_hint_label_description": "提示信息未加密并且可以从文件中读取！"
                                                          "\n不要使用容易被猜到的提示，例如"
                                                          "\n生日等。尝试使用不容易与您关联的参考。",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "输入提示（可选）",
    "dialog_encrypt_new_password_button_ok": "确定",
    "dialog_encrypt_new_password_button_cancel": "取消",
    "dialog_encrypt_new_password_warning_empty_title": "警告",
    "dialog_encrypt_new_password_warning_empty_text": "密码字段不能为空！",
    "dialog_encrypt_new_password_warning_too_long_title": "警告",
    "dialog_encrypt_new_password_warning_too_long_text": "提示字段太长，最多{symbols}个字符！",

    "dialog_encrypt_password_title": "输入密码",
    "dialog_encrypt_password_label": "密码：",
    "dialog_encrypt_password_input_placeholder_text": "输入密码",
    "dialog_encrypt_password_hint_label": "提示：",
    "dialog_encrypt_password_button_ok": "确定",
    "dialog_encrypt_password_button_cancel": "取消",

    "dialog_encrypt_password_reset_title": "重置加密密码",
    "dialog_encrypt_password_reset_text": "您确定要重置当前加密密码吗？",
    "dialog_encrypt_password_reset_button_cancel": "取消",
    "dialog_encrypt_password_reset_button_yes": "是",

    "dialog_open_link_title": "链接",
    "dialog_open_link_text": "在浏览器中打开链接 \"{url}\"?",

    "dialog_reset_settings_title": "重置设置？",
    "dialog_reset_settings_text": "将清除设置中的所有存储数据，并将重新启动应用程序以应用更改。",

    "dialog_exit_unsaved_title": "确认退出",
    "dialog_exit_unsaved_text": "无法保存已打开的文件 '{file_name}'。是否继续退出？",

    "message_app_config_file_access": "访问应用配置文件 {file_path} 时权限被拒绝。请设置正确的权限以确保正常操作。",

    "field_dir_path_dialog_caption": "选择目录",
    "field_file_path_dialog_caption": "选择文件",

    "load_file_encryption_password_mismatch": "加密密码不匹配!",
    "load_file_encryption_password_incorrect": "加密密码错误!",
    "load_file_none_content_error": "无法加载文件。",

    "action_new_file_error_occurred": "无法创建文件；发生错误。\n检查文件系统权限。",
    "save_active_file_error_occurred": "无法保存文件；发生错误。",

    "expandable_block_default_title": "更多信息...",
    "expandable_block_open_close_tags_mismatch_warning": "<details> 块打开/关闭标记不匹配",

    "dialog_color_picker_color_copied_to_the_clipboard": "格式化文本已复制到剪贴板",

    "popup_about_title": "应用信息",
    "popup_about_app_name_description": "Python Markdown 编辑器",

    "popup_about_version": "版本",
    "popup_about_license": "许可证",
    "popup_about_website": "网站",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "日期",

    "update_helper_new_version_is_available": "应用的新版本 {latest_version} 现已可用",
    "update_helper_latest_version_installed": "应用的最新版本已安装",

    "network_connection_error_empty": "无法获取响应信息",
    "network_connection_error_connection_or_dns":
        "未找到主机。可能是互联网连接或DNS出现问题。",
    "network_connection_error_connection_refused":
        "连接被拒绝。服务器可能已关闭或存在网络问题。",
    "network_connection_error_connection_timed_out": "连接超时。可能存在网络问题。",
    "network_connection_error_connection_404_error":
        "404连接错误。未找到请求的页面或资源。",
    "network_connection_error_generic_with_status_code": "请求失败，状态码：{status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QWidget functionality to provide horizontal UI spacer.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QWidget, QVBoxLayout, QFrame, QSpacerItem, QSizePolicy


class HorizontalLineSpacer(QWidget):
    """
    Vertical line separator for the status bar.
    """

    def __init__(self, parent=None):
        super(HorizontalLineSpacer, self).__init__(parent)

        # Main spacer layout
        main_layout = QVBoxLayout(self)

        # Add spacer above the line
        main_layout.addSpacerItem(QSpacerItem(0, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed))

        # Add horizontal delimiter
        line = QFrame(self)
        line.setFrameShape(QFrame.Shape.HLine)
        line.setFrameShadow(QFrame.Shadow.Sunken)
        main_layout.addWidget(line)

        # Add spacer below the line
        main_layout.addSpacerItem(QSpacerItem(0, 10, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed))



# Chinese lexemes common.py
lexemes = {
    "module_llama_cpp_name": "模块 llama.cpp",

    "module_llama_cpp_model_exception": "未找到模型错误：{error_msg}",
    "module_llama_cpp_task_exception": "无法初始化模型生成器：{error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the 'OpenAI API' module.
- Functionality: Enables model inference using supported APIs.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QVBoxLayout, QWidget, QLabel, QLineEdit, QSizePolicy, QPlainTextEdit, QSpinBox, QSlider
from PySide6.QtWidgets import QScrollArea
from PySide6.QtNetwork import QNetworkRequest, QNetworkAccessManager, QNetworkReply

import os
import json
import logging

import asyncio
from asyncio import Task
from qasync import asyncSlot

from typing import TYPE_CHECKING, Callable

from .api_helper import ApiHelper
from .prompt_manager import PromptManager

from ..base_ai_core import BaseAiCore

from .. import Settings
from .. import AppConfig
from .. import Lexemes

from ...enums.openai_model_names import OpenAiModelNames
from ...ui.enum_combo_box import EnumComboBox
from ...ui.ai_assistant.ai_assistant import EnumMessageType, EnumMessageStyle
from ...ui.horizontal_line_spacer import HorizontalLineSpacer
from ...ui.label_with_hint import LabelWithHint

if TYPE_CHECKING:
    from PySide6.QtCore import QByteArray  # noqa


class ModuleCore(BaseAiCore):
    update_signal = Signal(str, object, object, EnumMessageType, EnumMessageStyle)
    # prompt_tokens, completion_tokens, total_tokens
    update_usage_signal = Signal(str, object, object, object, object)

    # Default name, use lexeme for the translated one
    module_name = 'OpenAI API'

    # Functionality extended by the module.
    extensions = ['ai_assistant', 'settings_dialog']

    init_callback: Callable = None
    finished_callback: Callable = None

    generator_task: Task = None

    prompt_manager: PromptManager = None

    parent: None

    def __init__(self):
        super(ModuleCore, self).__init__()

        self.settings = Settings(parent=self)
        self.settings.value_changed.connect(
            lambda v: self.settings_update_handler(v))

        self.logger = logging.getLogger('module_openai_api')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language,
                               default_scope='settings_dialog',
                               lexemes_dir=self.get_lexemes_path())

        # Extend Settings explicitly as it's a global object which may cause circular recursion
        self.extend_settings_create_property(self.settings.create_property)
        # Add this module to the inference modules selection list
        inference_modules = self.settings.ai_config_inference_modules
        default_module = [key for key, value in inference_modules.items() if isinstance(value, tuple) and True in value]
        inference_modules.update({'openai_api': (self.lexemes.get('module_openai_api_name', scope='common'),
                                                 True if not default_module else False)})
        self.settings.ai_config_inference_modules = inference_modules

        # API requirements
        self.openai_api_url = self.settings.module_openai_api_url
        self.openai_api_key = self.settings.module_openai_api_key
        self.openai_api_model = OpenAiModelNames.__getitem__(self.settings.module_openai_api_model).value

        # Additional request params
        self.response_temperature = self.settings.module_openai_api_base_response_temperature
        self.response_max_tokens = self.settings.module_openai_api_base_response_max_tokens

        # API helper
        self.api_helper = ApiHelper()

        # Just in case of debug of async events
        # asyncio.get_event_loop().set_debug(True)

        self.logger.debug(f'Module {__name__} loaded')

    def init_prompt_manager(self, ai_dialog):
        """
        Prompt manager for prompt management and to add / append messages to prompt history.
        Note: It may not be initiated if called out of the context, from settings for example.
        """
        self.prompt_manager = PromptManager(
            system_prompt=self.settings.module_openai_api_base_system_prompt,
            max_history_size=self.settings.module_openai_api_prompt_history_size,
            parent=ai_dialog)

    def get_prompt_manager(self):
        return self.prompt_manager

    @staticmethod
    def get_lexemes_path():
        return os.path.join(os.path.dirname(__file__), 'lexemes')

    def attach_dialog(self, ai_dialog):
        # Catch parent dialog closing event
        ai_dialog.dialog_closed.connect(self.parent_closed)
        # Request cancelled event
        ai_dialog.request_cancelled.connect(self.request_cancelled)
        # Connect update signal to the dialog
        self.update_signal.connect(ai_dialog.append_to_message)
        # Connect to the token usage update signal
        self.update_usage_signal.connect(ai_dialog.update_usage)
        # Init and connect prompt manager
        self.init_prompt_manager(ai_dialog)

    @asyncSlot()
    async def parent_closed(self):
        ai_dialog = self.sender()
        # try/except because of an object might be deleted during processing callbacks asynchronously.
        # Explicitly check everything is disconnected.
        try:
            if ai_dialog:
                # Disconnect from the messages updating signal
                self.update_signal.disconnect(ai_dialog.append_to_message)
                # Disconnect from the token usage update signal
                self.update_usage_signal.disconnect(ai_dialog.update_usage)
                if hasattr(ai_dialog, 'dialog_closed'):
                    # Parent window might be closed
                    ai_dialog.dialog_closed.disconnect(self.parent_closed)
        except RuntimeError as e:  # Object might be already deleted
            self.logger.warning(f'Error occurred during the closing process {e}')
        # Clear prompt history
        PromptManager.reload()
        # Stop the generator thread before the dialog closes
        await self.stop_generator()

    @asyncSlot()
    async def request_cancelled(self):
        # Stop the generator thread before the dialog closes
        await self.stop_generator()

    @asyncSlot()
    async def request(self, user_prompt: str, request_msg_id: int, response_msg_id: int,
                      init_callback: Callable = None, finished_callback: Callable = None):

        if not asyncio.get_running_loop().is_running():
            return

        # Check the prompt manager is activated
        if not hasattr(self, 'prompt_manager') or self.prompt_manager is None:
            return

        # Store callbacks
        self.init_callback = init_callback
        self.finished_callback = finished_callback

        if not self.generator_task or self.generator_task.done():
            # self.generator_task = await asyncio.create_task(
            #    self.run_generator(user_prompt, request_msg_id, response_msg_id))
            self.generator_task = asyncio.ensure_future(
                self.run_generator(user_prompt, request_msg_id, response_msg_id))
            # Finished callback
            # This part will be completed in the handle_response() method after the async network reply.
            # if self.finished_callback and callable(self.finished_callback):
            #    self.generator_task.add_done_callback(
            #        lambda _task, _request_msg_id=request_msg_id, _response_msg_id=response_msg_id:
            #        self.finished_callback(request_msg_id=_request_msg_id, response_msg_id=_response_msg_id,
            #                               message_type=EnumMessageType.RESPONSE))
            # Run task asynchronously
            done, pending = await asyncio.wait([self.generator_task], return_when=asyncio.FIRST_COMPLETED)

            for task in done:
                if not task.cancelled():
                    exception = task.exception()
                    if exception:
                        self.logger.warning(f"Task raised an exception: {exception}")
                        # Error message to show in UI
                        outputs = self.lexemes.get('module_openai_api_task_exception', scope='common')
                        # Emit update message signal
                        self.update_signal.emit(outputs, None, None, EnumMessageType.DEFAULT,
                                                EnumMessageStyle.ERROR)
                    else:
                        result = task.result()
                        self.logger.debug(f"Task completed with result: {result}")

    async def run_generator(self, user_prompt, request_msg_id, response_msg_id):
        """
        Other params:
            "temperature": 0.2, "top_p": 1, "n": 1, "stream": False, ...
        """
        temperature_float = self.api_helper.convert_temperature(self.response_temperature)
        options = {'temperature': temperature_float}
        if self.response_max_tokens > 0:
            options.update({'max_tokens': self.response_max_tokens})
        # Request
        request = self.api_helper.init_request(api_url=self.openai_api_url, api_key=self.openai_api_key)
        # Request data
        request_data = self.api_helper.init_request_params(
            prompt_messages=user_prompt, api_model=self.openai_api_model, options=options)

        # Set request data
        manager = QNetworkAccessManager(self)
        reply = manager.post(request, request_data)

        # Connect finished signal
        reply.finished.connect(
            lambda _reply=reply: self.handle_response(_reply, request_msg_id, response_msg_id))
        # Connect error signal
        reply.errorOccurred.connect(
            lambda error, _reply=reply: self.handle_response(_reply, request_msg_id, response_msg_id, error))

    def handle_response(self, reply: QNetworkReply, request_msg_id, response_msg_id, error_code=None):
        # Get received status code, say 200
        status_code = reply.attribute(QNetworkRequest.Attribute.HttpStatusCodeAttribute)

        self.logger.debug(f'API response: {reply}, {error_code}, {status_code}')

        # Make sure there is no error
        if reply.error() == QNetworkReply.NetworkError.NoError:
            result_message = self.process_response(reply, request_msg_id, response_msg_id, status_code)
        elif reply.error() == QNetworkReply.NetworkError.HostNotFoundError:
            # The host was not found, indicating possible DNS issues or no internet connection
            result_message = self.lexemes.get('network_connection_error_connection_or_dns', scope='common')
        elif reply.error() == QNetworkReply.NetworkError.ConnectionRefusedError:
            # Connection was refused, indicating the server might be down or there are network issues
            result_message = self.lexemes.get('network_connection_error_connection_refused', scope='common')
        elif reply.error() == QNetworkReply.NetworkError.TimeoutError:
            # The connection timed out, indicating network issues
            result_message = self.lexemes.get('network_connection_error_connection_timed_out', scope='common')
        elif reply.error() == QNetworkReply.NetworkError.ContentNotFoundError:
            # The requested page or resource is not found
            result_message = self.lexemes.get('network_connection_error_connection_404_error', scope='common')
        # elif reply.error() == QNetworkReply.NetworkError.AuthenticationRequiredError:
        #    result_message = reply.errorString()
        else:
            # Handle other errors
            result_message = self.lexemes.get('network_connection_error_generic_with_status_code', scope='common',
                                              status_code=status_code)

        if error_code is not None:
            self.logger.warning(result_message)
            self.logger.warning(f"Failed to fetch information [{status_code}]: {reply.errorString()}")
            # Emit update message signal
            self.update_signal.emit(result_message if result_message else '[%s] %s' % (status_code, reply.errorString()),
                                    None, None, EnumMessageType.DEFAULT, EnumMessageStyle.ERROR)

        # Call API initialized callback here to update waiting status
        if self.init_callback and callable(self.init_callback):
            self.init_callback()

        # Call finished callback
        if self.finished_callback and callable(self.finished_callback):
            self.finished_callback(request_msg_id=request_msg_id, response_msg_id=response_msg_id,
                                   message_type=EnumMessageType.RESPONSE)

        reply.finished.disconnect()
        reply.errorOccurred.disconnect()
        reply.deleteLater()  # Clean up the QNetworkReply object

    def process_response(self, reply: QNetworkReply, request_msg_id, response_msg_id, status_code) -> str:
        # The request was successful
        data = reply.readAll()  # type: QByteArray

        self.logger.debug(f'Raw RESPONSE [{status_code}]: {data}')

        # json_document = QJsonDocument.fromJson(data)
        # json_data = json_document.object()

        # Convert QByteArray to string
        res_string = data.toStdString()  # Or: str(data.data(), encoding='utf-8')
        self.logger.debug(f'Result multi-line STRING: {res_string}')

        # Clean up the string and make one line
        json_str = ''.join(line.strip() for line in res_string.splitlines())

        self.logger.debug(f'Result STRING: {json_str}')

        result_message = self.lexemes.get('network_connection_error_empty', scope='common')
        # Parse JSON response
        try:
            json_data = json.loads(json_str)
            self.logger.debug(f"Result JSON: {json_data}")
            if ('choices' in json_data
                    and len(json_data['choices']) > 0
                    # Legacy completions
                    # and 'text' in json_data['choices'][0]):
                    and 'message' in json_data['choices'][0]
                    and 'content' in json_data['choices'][0]['message']):
                # Legacy completions
                # outputs = str(json_data['choices'][0]['text']).strip()
                outputs = str(json_data['choices'][0]['message']['content']).strip()
                # Emit update message signal
                self.update_signal.emit(outputs, request_msg_id, response_msg_id,
                                        EnumMessageType.RESPONSE, EnumMessageStyle.DEFAULT)
            if 'usage' in json_data:
                prompt_tokens, response_tokens, total_tokens = 0, 0, 0
                try:
                    # Ensure that all expected keys are present
                    keys = ('prompt_tokens', 'completion_tokens', 'total_tokens')
                    prompt_tokens, response_tokens, total_tokens = (json_data['usage'][key] for key in keys)
                except KeyError as e:
                    self.logger.warning(f"Missing key: {e}")
                except ValueError as e:
                    self.logger.warning(f"Value error: {e}")
                finally:
                    # Inference model
                    model = json_data['model'] if 'model' in json_data else None
                    # Emit update usage signal
                    self.update_usage_signal.emit(model, prompt_tokens, response_tokens, total_tokens, False)
        except json.JSONDecodeError as e:
            self.logger.warning("Error decoding JSON: %s" % e)

        return result_message

    async def stop_generator(self):
        # Cancel async task(s)
        if self.generator_task and not self.generator_task.done():
            # Allow to finish callback, do not remove:
            # > self.generator_task.remove_done_callback(self.finished_callback)
            self.generator_task.cancel()

    def extend_settings_dialog_fields_conf(self, tab_widget) -> list:
        # OpenAI API Config
        tab_openai_api_config = QWidget()

        # Create the scroll area
        scroll_area = QScrollArea()
        scroll_area.setObjectName('settings_dialog_tab_openai_api_config')
        scroll_area.setWidgetResizable(True)

        # Define the layout for the  OpenAI API configuration tab
        tab_openai_api_config_layout = QVBoxLayout(tab_openai_api_config)

        # Set the content widget inside the scroll area
        scroll_area.setWidget(tab_openai_api_config)

        tab_widget.addTab(scroll_area, self.lexemes.get('tab_openai_api_config'))

        return [
            # [OpenAI API config]
            # Label for the OpenAI API configuration block
            {"type": QLabel, "name": "settings_dialog_module_openai_api_label",
             "props": {"setProperty": ("class", "group-header-label")},
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_label'), "style": {"bold": True},
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # OpenAI API url label
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_url_input_accessible_description',
                            self.lexemes.get('module_openai_api_url_input_accessible_description'))},
             "name": "settings_dialog_module_openai_api_url_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_url_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # OpenAI API url line input
            {"type": QLineEdit, "name": "settings_dialog_module_openai_api_url:module_openai_api_url",
             "read_only": False, "max_length": 128,
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "placeholder_text": self.lexemes.get('module_openai_api_url_input_placeholder_text'),
             "accessible_description":
                 self.lexemes.get('module_openai_api_url_input_accessible_description')},
            # OpenAI API url label
            {"type": LabelWithHint, "kwargs": {"tooltip": ('module_openai_api_key_input_accessible_description',
                                               self.lexemes.get('module_openai_api_key_input_accessible_description'))},
             "name": "settings_dialog_module_openai_api_key_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_key_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # OpenAI API key line input
            {"type": QLineEdit, "name": "settings_dialog_module_openai_api_key:module_openai_api_key",
             "read_only": False, "max_length": 128, "props": {"setEchoMode": QLineEdit.EchoMode.Password},
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "placeholder_text": self.lexemes.get('module_openai_api_key_input_placeholder_text'),
             "accessible_description":
                 self.lexemes.get('module_openai_api_key_input_accessible_description')},
            # Vertical spacer
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Label for supported chat formats
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_model_names_combo_accessible_description',
                            self.lexemes.get('module_openai_api_model_names_combo_accessible_description'))},
             "name": "settings_dialog_module_openai_api_supported_models_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_supported_models_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Dropdown for selecting a supported chat format
            {"type": EnumComboBox, "args": [sorted(OpenAiModelNames, key=lambda member: member.legacy)],  # legacy below
             "name": "settings_dialog_module_openai_api_model_names_combo:module_openai_api_model",
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj),
             "placeholder_text": self.lexemes.get('module_openai_api_model_names_combo_placeholder_text'),
             "accessible_description":
                 self.lexemes.get('module_openai_api_model_names_combo_accessible_description')},
            # Horizontal spacer
            {"type": HorizontalLineSpacer, "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Label for the system prompt text editor
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_base_system_prompt_edit_accessible_description',
                            self.lexemes.get('module_openai_api_base_system_prompt_edit_accessible_description'))},
             "name": "settings_dialog_module_openai_api_base_system_prompt_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_base_system_prompt_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Text editor field for the system prompt
            {"type": QPlainTextEdit,
             "name": "settings_dialog_module_openai_api_base_system_prompt:"
                     "module_openai_api_base_system_prompt",
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "placeholder_text": self.lexemes.get('module_openai_api_base_system_prompt_edit_placeholder_text'),
             "accessible_description":
                 self.lexemes.get('module_openai_api_base_system_prompt_edit_accessible_description'), "text_lines": 7},
            # Vertical spacer
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Label for the temperature slider
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_base_response_temperature_input_accessible_description',
                            self.lexemes.get('module_openai_api_base_response_temperature_input_accessible_description'))},
             "name": "settings_dialog_module_openai_api_base_response_temperature_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_base_response_temperature_label',
                                      temperature=self.settings.module_openai_api_base_response_temperature),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Slider to adjust the temperature setting
            {"type": QSlider, "args": [Qt.Orientation.Horizontal],
             "props": {'setFocusPolicy': Qt.FocusPolicy.StrongFocus, 'setTickPosition': QSlider.TickPosition.TicksAbove,
                       'setTickInterval': 5, 'setSingleStep': 5, 'setMinimum': 0, 'setMaximum': 100},
             "name": "settings_dialog_module_openai_api_base_response_temperature:"
                     "module_openai_api_base_response_temperature",  # Lexeme key : Object name
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj),
             "on_value_change": self.temperature_change_handler,
             "accessible_description":
                 self.lexemes.get('module_openai_api_base_response_temperature_input_accessible_description')},
            # Vertical spacer
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Label for the maximum token count setting
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_base_response_max_tokens_input_accessible_description',
                            self.lexemes.get('module_openai_api_base_response_max_tokens_input_accessible_description'))},
             "name": "settings_dialog_module_openai_api_base_response_max_tokens_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_base_response_max_tokens_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Input field for the maximum token count setting
            {"type": QSpinBox, "props": {'setMinimum': 0, 'setMaximum': 65536},  # Update the highest range
             "size_policy": (QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred),
             "name": "settings_dialog_module_openai_api_base_response_max_tokens:"
                     "module_openai_api_base_response_max_tokens",
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "accessible_description":
                 self.lexemes.get('module_openai_api_base_response_max_tokens_input_accessible_description')},
            # Horizontal line spacer
            {"type": HorizontalLineSpacer, "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
            # Label for the prompt history maximum capacity settings
            {"type": LabelWithHint, "kwargs": {
                "tooltip": ('module_openai_api_config_prompt_history_size_input_accessible_description',
                            self.lexemes.get('module_openai_api_config_prompt_history_size_input_accessible_description'))},
             "name": "settings_dialog_module_openai_api_config_prompt_history_size_label",
             "alignment": Qt.AlignmentFlag.AlignLeft,
             "text": self.lexemes.get('module_openai_api_config_prompt_history_size_label'),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop)},
            # Input field for the prompt history maximum capacity setting
            {"type": QSpinBox, "props": {'setMinimum': 0, 'setMaximum': 65536},  # Update the highest range
             "size_policy": (QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred),
             "name": "settings_dialog_module_openai_api_config_prompt_history_size:"
                     "module_openai_api_prompt_history_size",
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj, alignment=Qt.AlignmentFlag.AlignTop),
             "accessible_description":
                 self.lexemes.get('module_openai_api_config_prompt_history_size_input_accessible_description')},
            # Spacer to align elements at the top of the layout
            {"type": QWidget, "name": None, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding),
             "callback": lambda obj: tab_openai_api_config_layout.addWidget(obj)},
        ]

    @staticmethod
    def extend_settings_create_property(extend_func: Callable):
        if callable(extend_func):
            extend_func("module_openai_api_url", str, "https://api.openai.com/v1/chat/completions")
            extend_func("module_openai_api_key", str, "", encrypt=True)
            extend_func("module_openai_api_model", str, str(OpenAiModelNames.default()))

            extend_func("module_openai_api_base_system_prompt", str, "")
            extend_func("module_openai_api_base_response_temperature", int, 20)
            extend_func("module_openai_api_base_response_max_tokens", int, 0)
            extend_func("module_openai_api_prompt_history_size", int, 0)

    def settings_update_handler(self, data) -> None:
        """
        Perform actions upon settings change.
        Data comes in a view of a dictionary, where is the key is the setting name, and the value is the actual value.
        Can be resource greedy.

        Args:
            data dict: say {"module_openai_api_model": "..."}

        Return:
            None
        """

        AppConfig().logger.debug('Settings update handler is in use "%s"' % data)

        options = [
            'module_openai_api_url',
            'module_openai_api_key',
            'module_openai_api_model',
            'module_openai_api_base_system_prompt',
            'module_openai_api_base_response_temperature',
            'module_openai_api_base_response_max_tokens',
        ]
        if any(option in data for option in options) or 'ai_config_inference_module' in data:
            pass

    def temperature_change_handler(self, source_object, source_widget):
        if source_object.objectName() and source_widget:
            # Parse the object name in case it contains a combination of lexeme and setting keys
            _lexeme_key, setting_name = self.settings.settings_helper.parse_object_name(source_object.objectName())
            if setting_name == 'module_openai_api_base_response_temperature':
                # Get the latest value from the object, as settings might not yet be updated
                temperature_int = source_object.value()  # source_object type: QSlider
                temperature_float = self.api_helper.convert_temperature(temperature_int)
                temperature_label = source_widget.findChild(
                    QLabel,
                    'settings_dialog_module_openai_api_base_response_temperature_label'
                )
                if temperature_label:
                    temperature_label.setText(
                        self.lexemes.get('module_openai_api_base_response_temperature_label',
                                         temperature=temperature_float))



# German lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "Modul llama.cpp",

    "module_llama_cpp_config_label": "Modul llama.cpp",
    "module_llama_cpp_config_path_label": "Modellstandort",
    "module_llama_cpp_config_path_input_placeholder_text": "Modellpfad auswählen oder eingeben",
    "module_llama_cpp_config_path_input_accessible_description":
        "Ein Eingabefeld mit einem Selektor, um den Pfad des lokalen Modells anzugeben.\n"
        "Unterstützt Modelle im GGUF-Format, ein binäres Dateiformat,\n"
        "optimiert zur Speicherung von Modellen, die mit GGML und GGML-basierten Ausführern verwendet werden.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF-Dateien",

    "module_llama_cpp_config_context_window_label": "Kontextfenstergröße",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "Legt die Anzahl der Tokens fest, die das Modell für die Erzeugung von Antworten berücksichtigt.\n"
        "Steuert, wie viel vorheriger Kontext verwendet wird.",

    "module_llama_cpp_chat_formats_label": "Chat-Formate",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "Chat-Format auswählen",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "Dropdown-Menü zur Auswahl des Formats, das für Modellgespräche verwendet wird.",

    "module_llama_cpp_config_system_prompt_label": "Systemaufforderung",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "Systemaufforderungstext eingeben",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "Textfeld für das Eingeben von Systemaufforderungen, die die Modellantworten leiten.",

    "module_llama_cpp_config_response_temperature_label": "Antworttemperatur: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "Passt die Zufälligkeit der Modellantworten an. Höhere Werte erzeugen variablere Ausgaben,\n"
        "während niedrigere Werte zu vorhersehbareren Antworten führen.",

    "module_llama_cpp_config_response_max_tokens_label": "Maximale Tokenanzahl pro Antwort",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "Begrenzt die Anzahl der Tokens in den Modellantworten bis zur tatsächlichen Grenze des Kontextfensters.\n"
        "Ein Wert von null nimmt die Kapazität des Kontextfensters an.",

    "module_llama_cpp_config_prompt_history_size_label": "Größe des Aufforderungsverlaufs",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "Steuer die Anzahl der Einträge im Aufforderungsverlauf, die vom System zur Referenz aufbewahrt werden.\n"
        "Ein Wert von null erlaubt eine unbegrenzte Anzahl von Einträgen."
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Part of the base module.
- Functionality: Facilitates access to the module's API functions.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import QObject

from abc import abstractmethod

from typing import Union


class BaseCore(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.parent = parent

    @property
    @abstractmethod
    def extensions(self) -> list:
        pass

    @property
    @abstractmethod
    def module_name(self):
        pass

    def get_name(self) -> str:
        return str(self)

    @staticmethod
    @abstractmethod
    def get_lexemes_path() -> Union[str, None]:
        # Subclasses are expected to override this method.
        raise NotImplementedError("Subclasses must implement get_lexemes_path()")

    def __str__(self) -> str:
        """
        Return module's name.
        """
        return self.module_name



# Italian lexemes common.py
lexemes = {
    "app_title": "Editor Notolog",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Campo filtro file",

    "menu_action_copy_file_path": "Copia il percorso del file",
    "menu_action_rename": "Rinomina",
    "menu_action_delete": "Elimina",
    "menu_action_delete_completely": "Elimina completamente",
    "menu_action_restore": "Ripristina",
    "menu_action_create_new_dir": "Crea una nuova directory",

    "dialog_file_rename_title": "Rinomina file",
    "dialog_file_rename_field_label": "Inserisci nuovo nome file",
    "dialog_file_rename_button_ok": "Rinomina",
    "dialog_file_rename_warning_exists": "Esiste già un file con lo stesso nome",

    "dialog_file_delete_title": "Elimina file",
    "dialog_file_delete_text": "Eliminare il file \"{file_name}\"?",
    "dialog_file_delete_completely_title": "Elimina file definitivamente",
    "dialog_file_delete_completely_text": "Eliminare definitivamente il file \"{file_name}\"?",
    "dialog_file_delete_error": "Impossibile eliminare il file, si è verificato un errore",
    "dialog_file_delete_error_not_found": "File non trovato",

    "dialog_file_restore_title": "Ripristina file",
    "dialog_file_restore_text": "Ripristinare il file \"{file_name}\"?",
    "dialog_file_restore_error": "Impossibile ripristinare il file, si è verificato un errore",
    "dialog_file_restore_warning_exists": "Esiste già un file con il nome {file_name}",

    "dialog_create_new_dir_title": "Crea una nuova directory",
    "dialog_create_new_dir_label": "Nome della nuova directory",
    "dialog_create_new_dir_input_placeholder_text": "Inserisci il nome della directory",
    "dialog_create_new_dir_button_ok": "Crea",
    "dialog_create_new_dir_button_cancel": "Annulla",
    "dialog_create_new_dir_warning_empty_title": "Errore nel nome della nuova directory",
    "dialog_create_new_dir_warning_empty_text": "Il nome della directory non può essere vuoto",
    "dialog_create_new_dir_warning_too_long_title": "Errore nel nome della nuova directory",
    "dialog_create_new_dir_warning_too_long_text": "Il nome della directory è troppo lungo; massimo "
                                                   "{symbols} caratteri consentiti!",
    "dialog_create_new_dir_error_existed": "La directory esiste già",
    "dialog_create_new_dir_error": "Impossibile creare la directory. Assicurati che la directory di destinazione "
                                   "{base_dir} sia scrivibile",

    "dialog_message_box_title": "Messaggio",
    "dialog_message_box_button_ok": "Chiudi",

    "action_new_file_first_line_template_text": "Nuovo documento",
    "action_open_file_dialog_caption": "Apri file",
    "action_save_as_file_dialog_caption": "Salva file",

    "dialog_save_empty_file_title": "Salva file vuoto",
    "dialog_save_empty_file_text": "Consentire di salvare il file senza contenuto?",

    "dialog_encrypt_file_title": "Cripta file",
    "dialog_encrypt_file_text": "Criptare il file \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "Il file è già criptato!",
    "dialog_encrypt_file_rewrite_existing_title": "Sovrascrivi file esistente",
    "dialog_encrypt_file_rewrite_existing_text": "Sovrascrivere il file esistente \"{file_path}\"?",

    "dialog_decrypt_file_title": "Decripta file",
    "dialog_decrypt_file_text": "Decriptare il file \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "Il file non è criptato!",
    "dialog_decrypt_file_rewrite_existing_title": "Sovrascrivi file esistente",
    "dialog_decrypt_file_rewrite_existing_text": "Sovrascrivere il file esistente \"{file_path}\"?",

    "dialog_encrypt_new_password_title": "Nuova password",
    "dialog_encrypt_new_password_label": "Password:",
    "dialog_encrypt_new_password_input_placeholder_text": "Inserisci la nuova password",
    "dialog_encrypt_new_password_hint_label": "Suggerimento:",
    "dialog_encrypt_new_password_hint_label_description":
        "Il suggerimento non è criptato e può essere letto dal file!"
        "\nNon utilizzare suggerimenti ovvi che possono essere indovinati, come"
        "\nla data di nascita, ecc. Prova a usare un riferimento.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Inserisci un suggerimento (opzionale)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Annulla",
    "dialog_encrypt_new_password_warning_empty_title": "Avviso",
    "dialog_encrypt_new_password_warning_empty_text": "Il campo della password non può essere vuoto!",
    "dialog_encrypt_new_password_warning_too_long_title": "Avviso",
    "dialog_encrypt_new_password_warning_too_long_text":
        "Il campo del suggerimento è troppo lungo, massimo {symbols} caratteri!",

    "dialog_encrypt_password_title": "Inserisci password",
    "dialog_encrypt_password_label": "Password:",
    "dialog_encrypt_password_input_placeholder_text": "Inserisci password",
    "dialog_encrypt_password_hint_label": "Suggerimento:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Annulla",

    "dialog_encrypt_password_reset_title": "Reimposta password di criptazione",
    "dialog_encrypt_password_reset_text": "Sei sicuro di voler reimpostare la password di criptazione attuale?",
    "dialog_encrypt_password_reset_button_cancel": "Annulla",
    "dialog_encrypt_password_reset_button_yes": "Sì",

    "dialog_open_link_title": "Link",
    "dialog_open_link_text": "Aprire il link \"{url}\" nel browser?",

    "dialog_reset_settings_title": "Reimposta impostazioni?",
    "dialog_reset_settings_text":
        "Tutti i dati memorizzati nelle impostazioni verranno cancellati, e l'app verrà riavviata per applicare "
        "le modifiche.",

    "dialog_exit_unsaved_title": "Conferma Uscita",
    "dialog_exit_unsaved_text": "Il file aperto '{file_name}' non può essere salvato. Continuare con l'uscita?",

    "message_app_config_file_access": "Permesso negato durante l'accesso al file di configurazione dell'applicazione in "
                                      "{file_path}. Imposta i permessi corretti per garantire un funzionamento adeguato.",

    "field_dir_path_dialog_caption": "Seleziona Directory",
    "field_file_path_dialog_caption": "Seleziona File",

    "load_file_encryption_password_mismatch": "La password di crittografia non corrisponde!",
    "load_file_encryption_password_incorrect": "Password di crittografia incorretta!",
    "load_file_none_content_error": "Impossibile caricare il file.",

    "action_new_file_error_occurred": "Impossibile creare il file; si è verificato un errore."
                                      "\nControllare le autorizzazioni del file system.",
    "save_active_file_error_occurred": "Impossibile salvare il file; si è verificato un errore.",

    "expandable_block_default_title": "Ulteriori informazioni...",
    "expandable_block_open_close_tags_mismatch_warning":
        "Incompatibilità delle tag di apertura/chiusura del blocco <details>",

    "dialog_color_picker_color_copied_to_the_clipboard": "Il testo formattato è stato copiato negli appunti",

    "popup_about_title": "Informazioni sull'Applicazione",
    "popup_about_app_name_description": "Editor Markdown Python",

    "popup_about_version": "Versione",
    "popup_about_license": "Licenza",
    "popup_about_website": "Sito Web",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Data",

    "update_helper_new_version_is_available": "Nuova versione {latest_version} dell'app disponibile",
    "update_helper_latest_version_installed": "L'ultima versione dell'app è installata",

    "network_connection_error_empty": "Impossibile ottenere informazioni sulla risposta",
    "network_connection_error_connection_or_dns":
        "Host non trovato. Potrebbe esserci un problema con la connessione internet o il DNS.",
    "network_connection_error_connection_refused":
        "Connessione rifiutata. Il server potrebbe essere non operativo o ci sono problemi di rete.",
    "network_connection_error_connection_timed_out": "Connessione scaduta. Potrebbero esserci problemi di rete.",
    "network_connection_error_connection_404_error":
        "Errore di connessione 404. La pagina o risorsa richiesta non è stata trovata.",
    "network_connection_error_generic_with_status_code": "Richiesta fallita con codice di stato: {status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Exposes package-level imports for easier access to submodules.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from ..app_config import AppConfig



# Korean lexemes main_menu.py
lexemes = {
    "group_file_label": "파일",
    "group_edit_label": "편집",
    "group_tools_label": "도구",
    "group_help_label": "도움말",

    "actions_file_label_new_document": "새 문서",
    "actions_file_accessible_name_new_document": "새 문서",
    "actions_file_label_open": "열기",
    "actions_file_accessible_name_open": "문서 열기",
    "actions_file_label_save": "저장",
    "actions_file_accessible_name_save": "문서 저장",
    "actions_file_label_save_as": "다른 이름으로 저장",
    "actions_file_accessible_name_save_as": "문서 복사본 저장",
    "actions_file_label_settings": "설정",
    "actions_file_accessible_name_settings": "앱 설정",
    "actions_file_label_reset_settings": "설정 재설정",
    "actions_file_accessible_name_reset_settings": "설정 재설정",
    "actions_file_label_exit": "종료",
    "actions_file_accessible_name_exit": "앱 종료",

    "actions_edit_edit_mode": "편집",
    "actions_edit_accessible_name_edit_mode": "파일 편집",
    "actions_edit_source_mode": "소스",
    "actions_edit_accessible_name_source_mode": "소스 코드",
    "actions_edit_bold": "굵게",
    "actions_edit_accessible_name_bold": "굵은 텍스트 형식",
    "actions_edit_italic": "기울임꼴",
    "actions_edit_accessible_name_italic": "기울임꼴 텍스트 형식",
    "actions_edit_underline": "밑줄",
    "actions_edit_accessible_name_underline": "밑줄 텍스트 형식",
    "actions_edit_strikethrough": "취소선",
    "actions_edit_accessible_name_strikethrough": "취소선 텍스트 형식",
    "actions_edit_blockquote": "블록 인용",
    "actions_edit_accessible_name_blockquote": "블록 인용 텍스트 형식",

    "actions_tools_ai_assistant": "AI 보조",
    "actions_tools_accessible_name_ai_assistant": "AI 보조 시작",
    "actions_tools_color_picker": "색상 선택기",
    "actions_tools_accessible_name_color_picker": "텍스트 색상 선택",

    "actions_help_label_md_syntax": "Markdown 문법",
    "actions_help_accessible_name_md_syntax": "Markdown 문법",
    "actions_help_label_check_for_updates": "업데이트 확인",
    "actions_help_accessible_name_check_for_updates": "업데이트 확인",
    "actions_help_label_bug_report": "버그 보고",
    "actions_help_accessible_name_bug_report": "버그 보고서 제출",
    "actions_help_label_about": "정보",
    "actions_help_accessible_name_about": "정보",
}



# Georgian lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "API-ს URL",
    "module_openai_api_url_input_placeholder_text": "API-ს URL",
    "module_openai_api_url_input_accessible_description":
        "OpenAI API-ს URL არის API-ის საბოლოო წერტილის მისამართი, რომელიც შეიძლება შეიცვალოს სერვისის და ვერსიის მიხედვით.\n"
        "AI ასისტენტი იყენებს მას საუბრის ჩატისთვის ან ტექსტის შევსებისთვის.\n"
        "მიმდინარე URL-ის მისაღებად მიმართეთ OpenAI API-ის ოფიციალურ დოკუმენტაციას.",
    "module_openai_api_key_label": "API გასაღები",
    "module_openai_api_key_input_placeholder_text": "API გასაღები",
    "module_openai_api_key_input_accessible_description":
        "OpenAI API გასაღები არის საიდუმლო ნიშანი, რომელიც გამოიყენება API-ის საბოლოო წერტილზე\n"
        "მოთხოვნების აუთენტიფიკაციისთვის.",
    "module_openai_api_supported_models_label": "მხარდაჭერილი მოდელები",
    "module_openai_api_model_names_combo_placeholder_text": "აირჩიეთ მოდელი",
    "module_openai_api_model_names_combo_accessible_description":
        "აირჩიეთ ჩატის საუბარებისთვის მხარდაჭერილი მოდელებიდან.",

    "module_openai_api_base_system_prompt_label": "სისტემის პრომპტი",
    "module_openai_api_base_system_prompt_edit_placeholder_text":
        "ბაზისის სისტემური პრომპტი, რომელიც წინასწარმოიდგინება თითოეულ მოთხოვნას",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "ბაზისის სისტემური პრომპტი, რომელიც წინასწარმოიდგინება თითოეულ მოთხოვნას.\n"
        "ჩვეულებრივ წერტილიანი ტექსტია, რომელშიც არის ინსტრუქციები ან როლის თვისებები.",

    "module_openai_api_base_response_temperature_label": "ტემპერატურა: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "რეგულირებს მოდელის პასუხების შემთხვევითობას. მაღალი მნიშვნელობები წარმოქმნიან უფრო რაზმიკებულ შედეგებს, "
        "ხოლო დაბალი მნიშვნელობები ხდის პასუხებს უფრო წინასწარ განსაზღვრად.",

    "module_openai_api_base_response_max_tokens_label": "პასუხის მაქსიმუმი ტოკენების რაოდენობა",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "მაქსიმალური ტოკენების რაოდენობას ანიჭებს, რომელიც პასუხში მიიღება, როგორიცაა სიტყვები და პუნქტუაცია, "
        "რეგულირებს შედეგის სიგრძეს.",

    "module_openai_api_config_prompt_history_size_label": "პრომპტის ისტორიის ზომა",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "კონტროლებს სისტემაში შენახული პრომპტის ისტორიის შენახვის შენახულ ჩანაწერების რაოდენობას.\n"
        "ნულოვანი მნიშვნელობა გამოიყენება შეუზღუდავ ჩანაწერებისთვის."
}
